function [A,X,U,info] = SBSSvecDC_alg(scD,r,prior,start,iterations)
% S-BSS-vecDC algorithm

    [p,n] = size(scD);

% prior parameters
    alpha0 = prior.alpha0;
    beta0 = prior.beta0;
    zeta0 = prior.zeta0;
    eta0 = prior.eta0;
    vartheta0 = prior.vartheta0;
    rho0 = prior.rho0;
    phi0 = prior.phi0;
    psi0 = prior.psi0;

% starting parameters
    hat_A = start.A;
    hat_AtA = start.AtA;
    hat_X = start.X;
    hat_XtX = start.XtX;
    hat_b = start.b;
    hat_btb = start.btb;
    hat_bbt = start.bbt;
    hat_vecU = start.vecU;
    
    hat_upsilon = start.upsilon;
    hat_Xi = start.Xi;
    hat_omega = start.omega;
    hat_varsigma = start.varsigma;

    % odvozene...
    varXX = 0.1*hat_X; % !!!!!!!!!!!!!!!!!!!!
    hat_B = hatB(hat_b);
    hat_BtB = hatBtB(hat_bbt);

% size setting
    hat_AA = zeros(p,r);
    varAA = zeros(p,r);
    mu_A = zeros(p,r);
    si_A = zeros(r,r,p);
    hat_vecUU = zeros(n*r,1);
    hat_bb = zeros(n,1);
    DEBUG_upsilon = zeros(iterations,r);
    DEBUG_Xi = zeros(iterations,r);
    DEBUG_ard = zeros(iterations,r);

konec = 1; iter = 0;
while konec
    iter = iter + 1;
    display(['S-BSS-vecDC: iteration ' num2str(iter) '/' num2str(iterations)]);
        
    
    % A
    % si_A
    for i = 1:p
        si_A(:,:,i) = ( hat_omega*hat_XtX + diag(hat_Xi(i,:)))^(-1);
    end  
    % mu_A
    for i = 1:p
        pom_sum_xdt = sum(hat_X.*kron(scD(i,:)',ones(1,r)),1);
        pom_sum_xdt = pom_sum_xdt';
        %
        mu_A(i,:) = ( si_A(:,:,i) * ( hat_omega*pom_sum_xdt ) )';
    end    
    % moments
    diagSiA = zeros(p*r,1);
    for k = 0:(r-1)
        diagSiA((1+p*k):((k+1)*p)) = si_A(k+1,k+1,:);
    end
    [hat_A(:) hat_AA(:)] = momtrun(mu_A(:),sqrt(diagSiA),0,1);
    varAA = hat_AA - hat_A.^2;
    hat_AtA = hat_A'*hat_A + diag(sum(varAA));  
    
    % Xi
    for i=1:p
        pom_ata = hat_A(i,:)'*hat_A(i,:) + diag(varAA(i,:));
        phi_i = phi0 + (1/2)*ones(1,r);
        psi_i = psi0 + (1/2)*diag(pom_ata)';
        hat_Xi(i,:) = phi_i./psi_i;
    end
    DEBUG_Xi(iter,:) = sum(hat_Xi);
    
    
    % U
    % si_vecU
    if 1 % psedoinverse
        toler = 10; 
        pom1 = (kron(hat_AtA,hat_omega*hat_BtB) + diag(hat_upsilon(:)));
        si_vecU = our_pinv(pom1,toler);
        
        toler = 10;
        pom2 = our_pinv(hat_BtB,toler);
        pom_vec = pom2*hat_B'*scD'*hat_A*hat_AtA^(-1);
        mu_vecU = si_vecU*(kron(hat_AtA,hat_omega*hat_BtB)*pom_vec(:));
    else
        si_vecU = (kron(hat_AtA,hat_omega*hat_BtB) + diag(hat_upsilon(:)))^(-1);
        pom_vec = hat_BtB^(-1)*hat_B'*scD'*hat_A*hat_AtA^(-1);
        mu_vecU = si_vecU*(kron(hat_AtA,hat_omega*hat_BtB)*pom_vec(:));
    end
    % moments
    [hat_vecU(:) hat_vecUU(:)] = momtrun_low(mu_vecU,sqrt(diag(si_vecU)),0);
    varvecUU = hat_vecUU - hat_vecU.^2;
    hat_vecUvecUt = diag(varvecUU) + hat_vecU*hat_vecU';
    if 1
        old_dSi = diag(si_vecU);
        new_dSi = varvecUU;
        clen = diag(sqrt(new_dSi./old_dSi));
        new_si_vecU = clen*si_vecU*clen;
        hat_vecUvecUt = new_si_vecU + hat_vecU*hat_vecU';
    end 
    hat_U = reshape(hat_vecU,n,r);
    
    
    % upsilon
    alpha = alpha0 + (1/2)*ones(n*r,1);
    beta = beta0 + (1/2)*diag(hat_vecUvecUt);
    hat_upsilon = alpha./beta;
    for k=0:(r-1)
        DEBUG_upsilon(iter,k+1) = sum(hat_upsilon((1+n*k):((k+1)*n)));
    end
    
    
    % b
    [dvojsuma,ctyrsuma] = sumy(hat_vecU,hat_vecUvecUt,hat_A,hat_AtA,scD); 
    si_b = (hat_varsigma*eye(n) + hat_omega*ctyrsuma)^(-1);
    mu_b = si_b*hat_omega*dvojsuma;
    [hat_b hat_bb] = momtrun_low(mu_b,sqrt(diag( si_b )),0);
    varb = hat_bb - hat_b.^2;
    hat_btb = hat_b'*hat_b + sum(varb);
    hat_bbt = hat_b*hat_b' + diag(varb);
    % derived from b
    hat_B = hatB(hat_b);
    hat_BtB = hatBtB(hat_bbt);


    % varsigma
    zeta = zeta0 + (n/2);
    eta = eta0 + (1/2)*trace(hat_btb);
    hat_varsigma = zeta/eta;
    DEBUG_varsigma(iter) = hat_varsigma;

    
    % X
    hat_X = hat_B*hat_U;
    hat_XtX = OcekHod(hat_vecU,diag(varvecUU),r,hat_BtB);

    
    % omega
    vartheta = vartheta0 + ((p*n)/2);
    rho = rho0 + (1/2)*(trace(scD'*scD - scD'*hat_A*hat_X' - hat_X*hat_A'*scD) + trace(hat_AtA*hat_XtX));
    hat_omega = vartheta/rho;
    DEBUG_omega(iter) = hat_omega;
    
    
    % stoping rule
    if iter == iterations
        konec = 0;
    end  
    
end % while    


% return values
A = hat_A;
X = hat_X;
U = hat_U;
info.omega = DEBUG_omega;
info.b = hat_b;
info.Xi = hat_Xi;
info.upsilon = DEBUG_upsilon;
info.debugXi = DEBUG_Xi;
info.varsigma = DEBUG_varsigma;
info.hat_upsilon = hat_upsilon;

end


function [E] = OcekHod(mu,sigma,er,Z)
% function for moment E(X'ZX)

%[z1, z2] = size(Z);
n1 = size(mu,1); % n1 --> n*r
q = n1/er; % q --> n

EE = submat(sigma,q,Z);
M = reshape(mu,q,er);
E = EE + M'*Z*M;

end

function [SE] = submat(S,q,Z)
% matrix of traces of sub-matrices

[n2,m] = size(S); % n2 --> nr
fs = n2/q; % fs --> r ; q = n

SE = zeros(fs); % --> SE \in R^{r x r}
for i=1:fs,
	for j=1:fs,
		SE(i,j) = trace(S((i-1)*q + (1:q),(j-1)*q + (1:q))*Z);
	end
end
end


function [hatB] = hatB(mub)
n = size(mub,1);
hatB = zeros(n);

for k=0:(n-1)
    sour = [  ((k+1):n)'  (1:(n-k))'  ];
    for m=1:size(sour,1)
        hatB(sour(m,1),sour(m,2)) = hatB(sour(m,1),sour(m,2)) + mub(k+1);
    end
end

end

function [hatBtB] = hatBtB(Ebbt)
n = size(Ebbt,1);
hatBtB = zeros(n);

    for k=0:(n-1)
        for l=0:(n-1)
            if (k<=l)
                souradnice = [  ((abs(k-l)+1):1:(n-max(k,l))+abs(k-l))'   (1:1:(n-max(k,l)))'  ];
            else
                souradnice = [  (1:1:(n-max(k,l)))'  ((abs(k-l)+1):1:(n-max(k,l))+abs(k-l))'   ];
            end
            for m=1:size(souradnice,1)
                hatBtB(souradnice(m,1),souradnice(m,2)) = hatBtB(souradnice(m,1),souradnice(m,2)) + Ebbt(k+1,l+1);
            end
        end
    end
end

function [dvojsum,ctyrsum] = sumy(hatvecU,hatvecUUt,hatA,hatAtA,scD)
% function for double- and quater- sums in VB equations

[p,r] = size(hatA);
[n1,q] = size(hatvecU);
n = n1/r;

dvojsum = zeros(n,1);
ctyrsum = zeros(n,n);

mu_vecU = hatvecU;
EvecUvecUt = hatvecUUt;

% double-sum
for i=1:r,
    vnitrni_suma = zeros(n,n);
    for k=0:(n-1),
        
        sour = [  (1:(n-k))'  ((k+1):n)'  ];
        for m=1:size(sour,1)
            vnitrni_suma(sour(m,1),sour(m,2)) = vnitrni_suma(sour(m,1),sour(m,2)) + mu_vecU((i - 1)*n + (k + 1));
        end
    end
    dvojsum = dvojsum + vnitrni_suma*scD'*hatA(:,i);
end

% quater-sum
for i=1:r,
    for j=1:r,
        vnitrni_suma = zeros(n,n);
        for k=0:(n-1),
            for l=0:(n-1),
                
                if (k<=l)
                    souradnice = [  ((abs(k-l)+1):1:(n-max(k,l))+abs(k-l))'   (1:1:(n-max(k,l)))'  ];
                else
                    souradnice = [  (1:1:(n-max(k,l)))'  ((abs(k-l)+1):1:(n-max(k,l))+abs(k-l))'   ];
                end
                for m=1:size(souradnice,1)
                    vnitrni_suma(souradnice(m,1),souradnice(m,2)) = vnitrni_suma(souradnice(m,1),souradnice(m,2)) + (EvecUvecUt((j-1)*n+(k+1),(i-1)*n+(l+1)));
                end
            end
        end
        ctyrsum = ctyrsum + hatAtA(i,j)*vnitrni_suma;
    end
end
end



function [ delt ] = delta(n,k)
% auxiliary matrix \Delta

delt = zeros(n,n);
for i=1:n
    for j=1:n
        if i-j==k
            delt(i,j) = 1;
        end
    end
end
end


function [Mpinv] = our_pinv(M,toler)
% upgraded pseudoinverse

[u,s,v] = svd(M);

sv = diag(s);

meansv = mean(sv);
tolerance = meansv/toler;

Mpinv = pinv(M,tolerance);

end














